//Inclusão da biblioteca .h
//A biblioteca deve incluida logo no início do software
#include "OPX001_IO.h"

//Definição do construtor
//O construtor deve ser chamado logo após a inclusão da biblioteca
//Deve ser preenchido como segue: (observar que a quantidade de IOs deve ser referente a montagem do hardware)
//OPX001_IO OPX(18, 8, 4, 18, 4, 1); (nesse caso shield possui todos os opcionais)
OPX001_IO::OPX001_IO(int nDI, int nAI, int nAT, int nDO, int nAO, int eE2P){
  this->_nDI=nDI;     //Número de entradas digitais           (0-18)
  this->_nAI=nAI;     //Número de entradas analógicas         (0-8)
  this->_nAT=nAT;     //Número de entradas de temperatura     (0-4)
  this->_nDO=nDO;     //Número de saídas digitais             (0-18)
  this->_nAO=nAO;     //Número de saídas analógicas           (0-4)
  this->_eE2P=eE2P;   //Habilita memória E2P                  (0-1)
  _wdata1 = 255;
  _wdata2 = 255;
  _wdata3 = 255;
  _rdata = 0;
  _rdata1 = 0;
  _rdata2 = 0;
  _return_x = 0;
  _ai_state = 0;
  _return_ai[8];
  _return_at[4];
}

//A função setup configura todos os opcionais presentes no hardware (Deverão ser preenchidos devidamente no construtor)
//Deve ser chamada dentro da função Setup do sistema.
//Chamada como segue:
//OPX.setup();
void OPX001_IO::setup(){
  Wire.begin(19, 18);           //I2C
  pinMode(35, INPUT);           //X0
  pinMode(34, INPUT);           //X1
  pinMode(4, OUTPUT);           //Y16
  pinMode(23, OUTPUT);          //Y17
  if (_nDI > 2){
    writeI2C(26, 2, 255);       //X2 - X9
  }
  if (_nDI > 9){
    writeI2C(27, 2, 255);       //X10 - X17
  }
  if ((_nAI > 0) || (_nAT > 0)){
    writeI2C(28, 1, 0);         //analog (AI/AT)
    writeI2C(28, 3, 0);         //analog (AI/AT)
  }
  if (_nDO > 0){
    writeI2C(24, 1, 255);       //Y0 - Y7
    writeI2C(24, 3, 0);         //Y0 - Y7
  }
  if (_nDO > 8){
    writeI2C(25, 1, 255);       //Y8 - Y15
    writeI2C(25, 3, 0);         //Y8 - Y15
  }
  if (_nDO > 16){
    digitalWrite(4,LOW);        //Y16
    digitalWrite(23,LOW);       //Y17
  }
  //ledcSetup(Canal, Fraquência, Resolução);
  //ledcAttachPin(Pino, Canal);
  ledcSetup(0, 1000, 12);       //AO0
  ledcAttachPin(26, 0);         //AO0
  ledcSetup(1, 1000, 12);       //AO1
  ledcAttachPin(25, 1);         //AO1
  ledcSetup(2, 1000, 12);       //AO2
  ledcAttachPin(5, 2);          //AO2
  ledcSetup(3, 1000, 12);       //AO3
  ledcAttachPin(27, 3);         //AO3
}

//A função DI realiza a aquisição dos valores das entradas digitais
//Deve ser chamada dentro da função Loop do sistema
//A chamada da função retorna o valor da entrada requisitada
//A atualização das entradas X0 e X1 são imediatas por estarem diretamente vinculadas aos pinos do controlador
//Os valores das entradas de X2 a X17 são atualizados a cada chamada da função "poll()"
//Chamada como segue:
//Variável = OPX.DI(15); (nesse caso leitura da entrada X15)
bool OPX001_IO::DI(int xDI){
  this->_xDI=xDI;                   //X0 - X17
  if (_xDI == 0){
    _return_x = !digitalRead(35);
  }
  if (_xDI == 1){
    _return_x = !digitalRead(34);
  }
  if ((_xDI >= 2) && (_xDI <= 9)){
    _return_x = bitRead(_rdata1, 7 - (_xDI - 2));
  }
  if ((_xDI >= 10) && (_xDI <= 17)){
    _return_x = bitRead(_rdata2, 7 - (_xDI - 10));
  }
  if (xDI < 0){
    _return_x = 0;
  }
  if (xDI > 17){
    _return_x = 0;
  }
  return _return_x;
}

//A função AI realiza a aquisição dos valores das entradas analógicas
//Deve ser chamada dentro da função Loop do sistema
//A chamada da função retorna o valor da entrada requisitada
//Os valores das entradas de AI0 a AI7 são atualizados a cada chamada da função "poll()"
//Elas tem o valor de 0-10000 equivalentes a tensão de entrada 0-10V
//Chamada como segue:
//Variável = OPX.AI(3); (nesse caso leitura da entrada AI3)
int OPX001_IO::AI(int vAI){
  int ai_resposta = 0;
  this->_vAI=vAI;                   //AI0 - AI7
  if (_vAI == 0){
    ai_resposta = _return_ai[4];
  }
  if (_vAI == 1){
    ai_resposta = _return_ai[6];
  }
  if (_vAI == 2){
    ai_resposta = _return_ai[7];
  }
  if (_vAI == 3){
    ai_resposta = _return_ai[5];
  }
  if (_vAI == 4){
    ai_resposta = _return_ai[2];
  }
  if (_vAI == 5){
    ai_resposta = _return_ai[1];
  }
  if (_vAI == 6){
    ai_resposta = _return_ai[0];
  }
  if (_vAI == 7){
    ai_resposta = _return_ai[3];
  }
  return  ai_resposta;
}

//A função AT realiza a aquisição dos valores das entradas de temperatura
//Deve ser chamada dentro da função Loop do sistema
//A chamada da função retorna o valor da entrada requisitada
//Os valores das entradas de AT0 a AT3 são atualizados a cada chamada da função "poll()"
//Elas tem o valor de 0-4095. Devem ser calibradas para cada tipo de sensor
//Chamada como segue:
//Variável = OPX.AT(0); (nesse caso leitura da entrada AT0)
int OPX001_IO::AT(int vAT){
  int at_resposta = 0;
  this->_vAT=vAT;                   //AT0 - AT3
  if (_vAT == 0){
    at_resposta = _return_at[2];
  }
  if (_vAT == 1){
    at_resposta = _return_at[1];
  }
  if (_vAT == 2){
    at_resposta = _return_at[0];
  }
  if (_vAT == 3){
    at_resposta = _return_at[3];
  }
  return  at_resposta;
}

//A função DO realiza o acionamento das saídas digitais
//Deve ser chamada dentro da função Loop do sistema
//A chamada da função aciona a saída requisitada
//A atualização das saidas Y16 e Y17 são imediatas por estarem diretamente vinculadas aos pinos do controlador
//Os valores das saídas de Y0 a Y15 são atualizados a cada chamada da função "poll()"
//Chamada como segue:
//OPX.DO(7, 1); (nesse caso acionamento da saída Y7 com valor alto)
//OPX.DO(9, 0); (nesse caso acionamento da saída Y9 com valor baixo)
void OPX001_IO::DO(int yDO, bool vDO){
  this->_yDO=yDO;                     //Número da saída Y0-Y17
  this->_vDO=vDO;                     //Valor da saída 0-1
  if ((_yDO >= 0) && (_yDO <= 7)){
    bitWrite(_wdata1, _yDO, !_vDO);
  }
  if ((_yDO >= 8) && (_yDO <= 15)){
    bitWrite(_wdata2, (_yDO - 8), !_vDO);
  }
  if (_yDO == 16){
    digitalWrite(4,_vDO); //Y16
  }
  if (_yDO == 17){
    digitalWrite(23,_vDO); //Y17
  }
}

//A função AO realiza o controle das saídas analógicas
//Deve ser chamada dentro da função Loop do sistema
//A chamada da função controla o valor da saída requisitada
//A atualização das saidas AO0 a AO3 são imediatas por estarem diretamente vinculadas aos pinos do controlador
//Elas tem o valor de 0-10000 equivalentes a tensão de de saída 0-10V
//Chamada como segue:
//OPX.AO(2, 5700); (nesse caso controle da saída AO2 com 5,7V)
void OPX001_IO::AO(int yAO, int vAO){
  this->_yAO=yAO;                     //Número da saída AO0-AO3
  this->_vAO=vAO;                     //Valor da saída 0-10000
  if ((yAO == 0) && (vAO >= 0) && (vAO <= 10000)){
    ledcWrite(0, map(vAO,0,10000,0,3670));
  }
  if ((yAO == 1) && (vAO >= 0) && (vAO <= 10000)){
    ledcWrite(1, map(vAO,0,10000,0,3670));
  }
  if ((yAO == 2) && (vAO >= 0) && (vAO <= 10000)){
    ledcWrite(2, map(vAO,0,10000,0,3670));
  }
  if ((yAO == 3) && (vAO >= 0) && (vAO <= 10000)){
    ledcWrite(3, map(vAO,0,10000,0,3670));
  }
}

//A função RE2P realiza a leitura do valor armazenado em uma determinada posição de memória
//da expansão de memória E2P presente no hardware
//A função é executada imediatamente ao ser chamada por isso deve-se ter atenção ao utilizar para
//não impactar no desempenho do controle
//A memória possui 65536 posições de memória (bytes)
//Chamada como segue:
//Variável = OPX.RE2P(200); (nesse caso leitura da posição de memória 200)
byte OPX001_IO::RE2P(unsigned int rAddress){
  byte E2P_Data = 0;
  this->_rAddress=rAddress;                 //Endereço da memória 0-65536
  if (_eE2P == 1){
    Wire.beginTransmission(80);
    Wire.write(_rAddress >> 8);   
    Wire.write(_rAddress); 
    Wire.endTransmission();
  
    Wire.requestFrom(80,1);
    if (Wire.available()){
      E2P_Data = Wire.read();
    }   
    return E2P_Data;
  }
}

//A função WE2P realiza a escrita do valor desejado em uma determinada posição de memória
//da expansão de memória E2P presente no hardware
//A função é executada imediatamente ao ser chamada por isso deve-se ter atenção ao utilizar para
//não impactar no desempenho do controle
//A memória possui 65536 posições de memória (bytes)
//Chamada como segue:
//Variável = OPX.WE2P(137, Variável); (nesse caso lescreve o valor da "Variável" na posição 137)
void OPX001_IO::WE2P(unsigned int wAddress, byte wData){
  this->_wAddress=wAddress;                 //Endereço da memória 0-65536
  this->_wData=wData;                       //Valor a ser armazenado (byte)
  if (_eE2P == 1){
    Wire.beginTransmission(80);
    Wire.write(_wAddress >> 8);   // Byte mais significativo
    Wire.write(_wAddress);        // Byte menos significativo
    Wire.write(_wData);
    Wire.endTransmission();
    delay(5);
  }
}

//A função poll realiza a atualização dos IOs através do barramento de comunicação
//O número de vezes a ser utilizado em um ciclo de máquina depende da demanda do controle
//Trata-se de uma troca entre número de atualizações e tempo de ciclo
//Idealmente deve-se chamar uma vez por ciclo de máquina no início ou fim da função Loop
//Chamada como segue:
//Variável = OPX.poll();
void OPX001_IO::poll(){
  if (_nDI > 2){
    _rdata1 = readI2C(26, 0); //X2 - X9
  }
  if (_nDI > 9){
    _rdata2 = readI2C(27, 0); //X10 - X17
  }
  if ((_nAI > 0) || (_nAT > 0)){
    bitWrite(_wdata3, 0, bitRead(_ai_state, 0));
    bitWrite(_wdata3, 1, bitRead(_ai_state, 1));
    bitWrite(_wdata3, 2, bitRead(_ai_state, 2));
    bitWrite(_wdata3, 3, bitRead(_ai_state, 0));
    bitWrite(_wdata3, 4, bitRead(_ai_state, 1));
    bitWrite(_wdata3, 5, 0);
    writeI2C(28, 1, _wdata3); //Analog
    _return_ai[_ai_state] = map(analogRead(36),0,3750,0,10000);
    if (_ai_state > 3){
      _return_at[_ai_state - 4] = analogRead(39);
    }else{
      _return_at[_ai_state] = analogRead(39);
    }
    _ai_state++;
    if (_ai_state > 7){
      _ai_state = 0;
    }
  }
  if (_nDO > 0){
    writeI2C(24, 1, _wdata1); //Y0 - Y7
  }
  if (_nDO > 8){
    writeI2C(25, 1, _wdata2); //Y8 - Y15
  }
}

//Função interna para aquisição de valores no barramento de comunicação
byte OPX001_IO::readI2C(byte _devadd_r, byte _command_r){
  Wire.beginTransmission(_devadd_r);
  Wire.write(_command_r);
  Wire.endTransmission();
  Wire.requestFrom(_devadd_r, 1);
  if (Wire.available()) {
    _rdata = Wire.read();
  }
  return _rdata;
}

//Função interna para envio de valores no barramento de comunicação
void OPX001_IO::writeI2C(byte _devadd_w, byte _command_w, byte _data_w){
  Wire.beginTransmission(_devadd_w);
  Wire.write(_command_w);
  Wire.write(_data_w);
  Wire.endTransmission();
  delay(5);
}
