/*
Arquivo             OPX001_IO.h
Versão              V1.0
Data                17/04/2023
Autor               Alberto Degrecci
Contato             www.opller.com

Descrição:
  Essa biblioteca foi desenvolvida com o objetivo de facilitar a utilização
do shield industrial OPX001 da Opller Equipamentos Industriais.
  Como trata-se de um shield customizável, recomenda-se a configuração dos
I/Os conforme a montagem do hardware para otimização do barramento de 
comunicação.
  Usuários avançados podem utilizar essa biblioteca como base, modificando
a mesma para otimização dos controles.
*/

//Definição da chamada da biblioteca
#ifndef OPX001_IO_H_INCLUDED
#define OPX001_IO_H_INCLUDED

//Inclusão das bibliotecas necessárias ao controle
#include <Arduino.h>
#include <Wire.h>

//Criação da classe / construtor / Funções / Variáveis
class OPX001_IO{
  public:
    OPX001_IO(int nDI, int nAI, int nAT, int nDO, int nAO, int eE2P);
    void setup();
    bool DI(int xDI);
    int  AI(int vAI);
    int  AT(int vAT);
    void DO(int yDO, bool vDO);
    void AO(int yAO, int vAO);
    byte  RE2P(unsigned int rAddress);
    void WE2P(unsigned int wAddress, byte wData);
    void poll();
  private:
    void writeI2C(byte _devadd_w, byte _command_w, byte _data_w);
    byte readI2C(byte _devadd_r, byte _command_r);
    int _nDI;
    int _nAI;
    int _nAT;
    int _nDO;
    int _nAO;
    int _eE2P;
    int _xDI;
    int _vAI;
    int _vAT;
    int _yDO;
    int _vDO;
    int _yAO;
    int _vAO;
    byte _wdata1 = 255;
    byte _wdata2 = 255;
    byte _wdata3 = 255;
    byte _rdata = 0;
    byte _rdata1 = 0;
    byte _rdata2 = 0;
    bool _return_x = 0;
    int  _ai_state = 0;
    int _return_ai[8];
    int _return_at[4];
    unsigned int _rAddress;
    unsigned int _wAddress;
    byte _wData;
};

#endif //OPX001_IO_H_INCLUDED 